﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI.WebControls;
using viBlog.Classes;
using viBlog.Controllers;

public partial class Admin_Pages_viBlog_Posts_PostEditor : System.Web.UI.UserControl
{
    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            AUTHORS.DataSource = Membership.GetAllUsers();
            AUTHORS.DataBind();
            CATEGORIES.DataSource = CategoryController.GetCategories();
            CATEGORIES.DataTextField = "Name";
            CATEGORIES.DataValueField = "CategoryId";
            CATEGORIES.DataBind();
            TAGS.DataSource = TagController.GetTags();
            TAGS.DataTextField = "Name";
            TAGS.DataValueField = "TagId";
            TAGS.DataBind();
            //create a new post
            if (Request.QueryString["PostId"] == null)
            {
                DATE.Text = DateTime.Now.ToString();
            }
            else
            {// edit an already existing post
                int PostId = Convert.ToInt32(Request.QueryString["PostId"]);
                Post post = PostController.GetPost(PostId);
                TITLE.Text = post.Title;
                DATE.Text = post.Date.ToString();
                {
                    URL.Visible = true;
                    URL.Text = post.Url;
                }
                SUMMARY.Text = post.Summary;
                DESCRIPTION.Text = PostController.GetPostDescription(PostId);
                //categories
                List<Category> categories = CategoryController.GetCategories(post.PostId);
                if (categories != null)
                {
                    foreach (Category category in categories)
                    {
                        CATEGORIES.Items.FindByValue(category.CategoryId.ToString()).Selected = true;
                    }
                }
                //tags
                List<Tag> tags = TagController.GetTags(post.PostId);
                if (tags != null)
                {
                    foreach (Tag tag in tags)
                    {
                        TAGS.Items.FindByValue(tag.TagId.ToString()).Selected = true;
                    }
                }
            }
        }
    }

    protected void SAVE_Click(object sender, EventArgs e)
    {
        //save a new post
        if (Request.QueryString["PostId"] == null)
        {
            Post post = new Post();
            post.Title = TITLE.Text;
            post.Author = AUTHORS.SelectedValue;
            post.Date = Convert.ToDateTime(DATE.Text);
            post.Url = RemoveIllegalCharacters(post.Title);
            post.Summary = SUMMARY.Text;
            int postid = PostController.AddPost(post, DESCRIPTION.Text);
            //categories
            if (postid != 0)// add categories
            {
                foreach (ListItem category in CATEGORIES.Items)
                {
                    if (category.Selected)
                        CategoryController.InsertCategory(Convert.ToInt32(category.Value), postid);
                }
            }
            //tags
            if (postid != 0)// add tags
            {
                foreach (ListItem tag in TAGS.Items)
                {
                    if (tag.Selected)
                        TagController.InsertTag(Convert.ToInt32(tag.Value), postid);
                }
            }
        }
        else
        {// update an already existing post
            int postid = 0;
            if (!Int32.TryParse(Request.QueryString["PostId"], out postid))
                return;
            Post post = PostController.GetPost(postid);
            if (post == null)
                return;
            post.Title = TITLE.Text;
            post.Author = AUTHORS.SelectedValue;
            post.Date = Convert.ToDateTime(DATE.Text);
            post.Url = RemoveIllegalCharacters(post.Title);
            post.Summary = SUMMARY.Text;
            PostController.UpdatePost(post, DESCRIPTION.Text);

            List<Category> categories = CategoryController.GetCategories(post.PostId);
            if (categories != null)
            {
                foreach (ListItem category in CATEGORIES.Items)
                {
                    if (category.Selected)
                    {
                        int catid = Convert.ToInt32(category.Value);
                        if (!categories.Exists(p => p.CategoryId == catid))
                            CategoryController.InsertCategory(catid, postid);
                    }
                    else
                    {
                        int catid = Convert.ToInt32(category.Value);
                        if (categories.Exists(p => p.CategoryId == catid))
                            CategoryController.RemoveCategory(catid, postid);
                    }
                }
            }
            //tags
            List<Tag> tags = TagController.GetTags(postid);
            if (tags != null)
            {
                foreach (ListItem tag in TAGS.Items)
                {
                    if (tag.Selected)
                    {
                        int tagid = Convert.ToInt32(tag.Value);
                        if (!tags.Exists(p => p.TagId == tagid))
                            TagController.InsertTag(tagid, postid);
                    }
                    else
                    {
                        int tagid = Convert.ToInt32(tag.Value);
                        if (tags.Exists(p => p.TagId == tagid))
                            TagController.RemoveTag(tagid, postid);
                    }
                }
            }
        }
        Response.Redirect(Request.RawUrl);
    }

    protected void CANCEL_Click(object sender, EventArgs e)
    {
        Response.Redirect(Request.RawUrl);
    }
    
    private static string RemoveIllegalCharacters(string Title)
    {
        if (string.IsNullOrEmpty(Title))
            return Title;

        Title = Title.Replace(":", string.Empty);
        Title = Title.Replace("/", string.Empty);
        Title = Title.Replace("?", string.Empty);
        Title = Title.Replace("#", string.Empty);
        Title = Title.Replace("[", string.Empty);
        Title = Title.Replace("]", string.Empty);
        Title = Title.Replace("@", string.Empty);
        Title = Title.Replace("*", string.Empty);
        Title = Title.Replace(".", string.Empty);
        Title = Title.Replace(",", string.Empty);
        Title = Title.Replace("\"", string.Empty);
        Title = Title.Replace("&", string.Empty);
        Title = Title.Replace("'", string.Empty);
        Title = Title.Replace(" ", "-");
        return HttpUtility.UrlEncode(Title).Replace("%", string.Empty);
    }
}
