﻿using System;
using System.Collections.Generic;
using System.Web.UI;
using viBlog.Classes;
using viBlog.Controllers;
using visinia.SmartEngine.Modules;
using visinia.SmartEngine.Site;
using visinia.SmartEngine.Utility;

public partial class Modules_viBlog_PostsList_PostsList : WebModule
{
    protected void Page_Load(object sender, EventArgs e)
    {
        String csname = "viBlogPager";
        String csurl = ResolveUrl("~/ClientScripts/viBlog/pager.js");
        ClientScriptManager cs = Page.ClientScript;
        if (!cs.IsClientScriptIncludeRegistered(csname))
        {
            cs.RegisterClientScriptInclude(csname, csurl);
        }        
        // look for the client side pager command
        if(string.IsNullOrEmpty(Request.QueryString["QS"]))
            Bind();        
        else
            BindPage(Request.QueryString["QS"]);
    }

    private void BindPage(string qs)
    {
        // the client side pager command
        string[] array = qs.Split('/');
        if (array.Length > 2)
        {            
            if (array[1].ToLower() == "category")
            {
                string categoryName = TrimExtension(array[2]);
                Category category = CategoryController.GetCategory(categoryName);
                if (category != null)
                {
                    DisplayPostsByCategory(category);                
                }
            }
            if (array[1].ToLower() == "tag")
            {
                string tagName = TrimExtension(array[2]);
                Tag tag = TagController.GetTag(tagName);
                if (tag != null)
                {
                    DisplayPostsByTag(tag);                    
                }
            }
            if (array[1].ToLower() == "archive")
            {
                if (array.Length == 4)
                {
                    string yearStr = array[2];
                    int year = 0;
                    if (Int32.TryParse(yearStr, out year))
                    {
                        string monthStr = TrimExtension(array[3]);
                        int month = 0;
                        if (Int32.TryParse(monthStr, out month))
                        {
                            DateTime date = new DateTime(year, month, 1);
                            DisplayPostsByDate(date);                            
                        }
                    }
                }
            }
        }
        else
            Bind();
    }

    private string TrimExtension(string strWithExt)
    {
        if (strWithExt.EndsWith(PostController.viBlogExtension))
            return strWithExt.Remove(strWithExt.LastIndexOf('.'));
        else
            return strWithExt;
    }

    private void Bind()
    {
        if (!string.IsNullOrEmpty(Request.QueryString["PostId"]))
        {
            int postid = 0;
            if (Int32.TryParse(Request.QueryString["PostId"], out postid))
            {   // show full post                    
                DisplaySinglePost(PostController.GetPost(postid));
            }
        }
        else if (!string.IsNullOrEmpty(Request.QueryString["Category"]))
        {
            Category category = CategoryController.GetCategory(Request.QueryString["Category"]);
            if (category != null)
            {
                DisplayPostsByCategory(category);
                //DisplayPosts(PostController.GetPosts(category, BlogController.PostsCountPerPage));
            }
        }
        else if (!string.IsNullOrEmpty(Request.QueryString["Tag"]))
        {
            Tag tag = TagController.GetTag(Request.QueryString["Tag"]);
            if (tag != null)
            {
                DisplayPostsByTag(tag);
                //DisplayPosts(PostController.GetPosts(tag, BlogController.PostsCountPerPage));
            }
        }
        else if ((!string.IsNullOrEmpty(Request.QueryString["Year"])) && (!string.IsNullOrEmpty(Request.QueryString["Month"])))
        {
            int year = 0;
            if (Int32.TryParse(Request.QueryString["Year"], out year))
            {
                int month = 0;
                if (Int32.TryParse(Request.QueryString["Month"], out month))
                {
                    DateTime date = new DateTime(year, month, 1);
                    DisplayPostsByDate(date);
                    //DisplayPosts(PostController.GetPosts(date));
                }
            }
        }
        else
            DisplayLatestPosts();
    }

    private void DisplayPostsByDate(DateTime date)
    {
        string page = Request.QueryString["Page"];
        int CurrentPage = 0;
        Int32.TryParse(page, out CurrentPage);
        CurrentPageState.Value = CurrentPage.ToString();

        if (CurrentPage == 0)
            pager_Home.Visible = false;
        else
            pager_Home.Visible = true;

        int total = PostController.GetPostsCount(date);
        int pages = total / BlogController.PostsCountPerPage;
        if ((total % BlogController.PostsCountPerPage) > 0)
            ++pages;
        --pages;//to make pages as zero based                                    
        if (pages >= CurrentPage)
        {
            if (pages > CurrentPage)
                pager_Next.Visible = true;
            else
                pager_Next.Visible = false;
            if (CurrentPage > 0)
                pager_Prev.Visible = true;
            else
                pager_Prev.Visible = false;

            int AccessCount = (BlogController.PostsCountPerPage * CurrentPage) + BlogController.PostsCountPerPage;
            int DisplayedCount = BlogController.PostsCountPerPage * CurrentPage;
            List<Post> list = PostController.GetPosts(date);
            if (list.Count < AccessCount)
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, list.Count - DisplayedCount));
            else
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, BlogController.PostsCountPerPage));
        }
    }

    private void DisplayPostsByTag(Tag tag)
    {
        string page = Request.QueryString["Page"];
        int CurrentPage = 0;
        Int32.TryParse(page, out CurrentPage);
        CurrentPageState.Value = CurrentPage.ToString();

        if (CurrentPage == 0)
            pager_Home.Visible = false;
        else
            pager_Home.Visible = true;

        int total = PostController.GetPostsCount(tag);
        int pages = total / BlogController.PostsCountPerPage;
        if ((total % BlogController.PostsCountPerPage) > 0)
            ++pages;
        --pages;//to make pages as zero based                                    
        if (pages >= CurrentPage)
        {
            if (pages > CurrentPage)
                pager_Next.Visible = true;
            else
                pager_Next.Visible = false;
            if (CurrentPage > 0)
                pager_Prev.Visible = true;
            else
                pager_Prev.Visible = false;

            int AccessCount = (BlogController.PostsCountPerPage * CurrentPage) + BlogController.PostsCountPerPage;
            int DisplayedCount = BlogController.PostsCountPerPage * CurrentPage;
            List<Post> list = PostController.GetPosts(tag, AccessCount);
            if (list.Count < AccessCount)
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, list.Count - DisplayedCount));
            else
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, BlogController.PostsCountPerPage));
        }
    }

    private void DisplayPostsByCategory(Category category)
    {
        string page = Request.QueryString["Page"];
        int CurrentPage = 0;
        Int32.TryParse(page, out CurrentPage);
        CurrentPageState.Value = CurrentPage.ToString();
        
        if (CurrentPage == 0)
            pager_Home.Visible = false;
        else
            pager_Home.Visible = true;

        int total = PostController.GetPostsCount(category);
        int pages = total / BlogController.PostsCountPerPage;
        if ((total % BlogController.PostsCountPerPage) > 0)
            ++pages;
        --pages;//to make pages as zero based                                    
        if (pages >= CurrentPage)
        {
            if (pages > CurrentPage)
                pager_Next.Visible = true;
            else
                pager_Next.Visible = false;
            if (CurrentPage > 0)
                pager_Prev.Visible = true;
            else
                pager_Prev.Visible = false;            
            int AccessCount = (BlogController.PostsCountPerPage * CurrentPage) + BlogController.PostsCountPerPage;
            int DisplayedCount = BlogController.PostsCountPerPage * CurrentPage;
            List<Post> list = PostController.GetPosts(category, AccessCount);
            if (list.Count < AccessCount)
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, list.Count - DisplayedCount));
            else
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, BlogController.PostsCountPerPage));
        }
    }

    private void DisplayLatestPosts()
    {
        string page = Request.QueryString["Page"];
        int CurrentPage = 0;
        Int32.TryParse(page, out CurrentPage);
        CurrentPageState.Value = CurrentPage.ToString();

        if (CurrentPage == 0)
            pager_Home.Visible = false;
        else
            pager_Home.Visible = true;

        int total = PostController.GetPostsCount();
        int pages = total / BlogController.PostsCountPerPage;
        if ((total % BlogController.PostsCountPerPage) > 0)
            ++pages;
        --pages;//to make pages as zero based                                    
        if (pages >= CurrentPage)
        {
            if (pages > CurrentPage)
                pager_Next.Visible = true;
            else
                pager_Next.Visible = false;
            if (CurrentPage > 0)
                pager_Prev.Visible = true;
            else
                pager_Prev.Visible = false;

            int AccessCount = (BlogController.PostsCountPerPage * CurrentPage) + BlogController.PostsCountPerPage;
            int DisplayedCount = BlogController.PostsCountPerPage * CurrentPage;
            List<Post> list = PostController.GetPosts(AccessCount);
            if (list.Count < AccessCount)
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, list.Count - DisplayedCount));
            else
                DisplayPosts(list.GetRange(BlogController.PostsCountPerPage * CurrentPage, BlogController.PostsCountPerPage));
        }
    }

    private void DisplaySinglePost(Post post)
    {
        PostView view = (PostView)LoadControl(WebRoots.GetThemesRoot() + SiteSettings.Theme + "/viBlog/PostView.ascx");
        view.Post = post;
        view.ShowFullPost = true;
        view.ShowAds = true;
        view.Ads = (string)Settings["Ads"];
        POSTS_LIST.Controls.Add(view);
        
        // SEO            
        SeoController.AddPageTitle(Page, post.Title);
        SeoController.AddMetaTag(Page, SeoController.MetaTags.description, post.Summary);
        SeoController.AddMetaTag(Page, SeoController.MetaTags.author, post.Author);        
        List<string> tagNames = TagController.GetTags(post.PostId).ConvertAll(
                                new Converter<Tag, string>(GetTagName));
        string tagKeywords = string.Join(",", tagNames.ToArray());
        List<string> categoryNames = CategoryController.GetCategories(post.PostId).ConvertAll(
                                new Converter<Category, string>(GetCategoryName));
        string categoryKeywords = string.Join(",", categoryNames.ToArray());
        string keywords = "";
        if (string.IsNullOrEmpty(tagKeywords))
            keywords = categoryKeywords;
        else
            keywords = tagKeywords + "," + categoryKeywords;
        if ( ! string.IsNullOrEmpty(keywords))
            SeoController.AddMetaTag(Page, SeoController.MetaTags.keywords, keywords);       
    }

    private static string GetTagName(Tag tag)
    {
        return tag.Name;
    }

    private static string GetCategoryName(Category category)
    {
        return category.Name;
    }

    private void DisplayPosts(List<Post> posts)
    {
        POSTS_LIST.Controls.Clear();
        foreach (Post post in posts)
        {
            PostView view = (PostView)LoadControl(WebRoots.GetThemesRoot() + SiteSettings.Theme + "/viBlog/PostView.ascx");
            view.Post = post;
            POSTS_LIST.Controls.Add(view);
        }
    }
}
